SystemOrganization addCategory: #ProcessSpecific!
SystemOrganization addCategory: #'ProcessSpecific-Tests'!

!Process methodsFor: '*ProcessSpecific' stamp: 'mvl 3/13/2007 12:24'!
environmentAt: key 
	^self environmentAt: key ifAbsent:[nil]! !

!Process methodsFor: '*ProcessSpecific' stamp: 'mvl 3/13/2007 12:24'!
environmentAt: key  ifAbsent: aBlock
	env ifNil: [ ^ aBlock value ].
	^env at: key ifAbsent: aBlock.! !

!Process methodsFor: '*ProcessSpecific' stamp: 'mvl 3/13/2007 12:12'!
environmentAt: key put: value
	env ifNil: [ env _ Dictionary new ].
	env at: key put: value.! !

TestCase subclass: #ProcessSpecificTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ProcessSpecific-Tests'!

!ProcessSpecificTest commentStamp: 'mvl 3/13/2007 13:52' prior: 0!
A ProcessSpecificTest is a test case for process local and dynamic variables.
!

!ProcessSpecificTest methodsFor: 'testing' stamp: 'mvl 3/13/2007 14:13'!
checkDynamic: value
	self assert: TestDynamicVariable value = value! !

!ProcessSpecificTest methodsFor: 'testing' stamp: 'mvl 3/13/2007 14:13'!
checkLocal: value
	self assert: TestLocalVariable value = value! !

!ProcessSpecificTest methodsFor: 'testing' stamp: 'mvl 3/13/2007 15:02'!
testDynamicVariable

	| s1 s2 p1stopped p2stopped |
	s1 := Semaphore new.
	s2 := Semaphore new.
	p1stopped := p2stopped := false.
	[
		TestDynamicVariable value: 1 during:[
			self checkDynamic: 1.
			(Delay forMilliseconds: 30) wait.
			self checkDynamic: 1.
			TestDynamicVariable value: 3 during:[
				(Delay forMilliseconds: 30) wait.
				self checkDynamic: 3
			].
			self checkDynamic: 1.
		].
		self checkDynamic: nil.
		p1stopped := true.
		s1 signal.
	] fork.

	[
		TestDynamicVariable value: 2 during:[
			self checkDynamic: 2.
			(Delay forMilliseconds: 30) wait.
			self checkDynamic: 2.
		].
		self checkDynamic: nil.
		p2stopped := true.
		s2 signal.
	] fork.

	"Set a maximum wait timeout so that the test case will complete 
	 even if the processes fail to signal us."
	s1 waitTimeoutSeconds: 2.
	s2 waitTimeoutSeconds: 2.
	self assert: p1stopped.
	self assert: p2stopped.! !

!ProcessSpecificTest methodsFor: 'testing' stamp: 'mvl 3/13/2007 15:03'!
testLocalVariable

	| s1 s2 p1stopped p2stopped |
	s1 := Semaphore new.
	s2 := Semaphore new.
	p1stopped := p2stopped := false.
	[
		self checkLocal: 0.
		TestLocalVariable value: 1.
		self checkLocal: 1.
		(Delay forMilliseconds: 30) wait.
		self checkLocal: 1.
		TestLocalVariable value: 2.
		self checkLocal: 2.
		p1stopped := true.
		s1 signal.
	] fork.

	[
		(Delay forMilliseconds: 30) wait.
		self checkLocal: 0.
		TestLocalVariable value: 3.
		self checkLocal: 3.
		(Delay forMilliseconds: 30) wait.
		self checkLocal: 3.
		TestLocalVariable value: 4.
		self checkLocal: 4.
		p2stopped := true.
		s2 signal.
	] fork.

	"Set a maximum wait timeout so that the test case will complete 
	 even if the processes fail to signal us."
	s1 waitTimeoutMSecs: 5000.
	s2 waitTimeoutMSecs: 5000.
	self assert: p1stopped.
	self assert: p2stopped.
! !

Object subclass: #ProcessSpecificVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ProcessSpecific'!

ProcessSpecificVariable class
	instanceVariableNames: 'hash'!

!ProcessSpecificVariable commentStamp: 'mvl 3/13/2007 13:53' prior: 0!
My subclasses (not instances of them) keep state specific to the current process.

There are two kinds of process-specific variables: process-local (state available
for read and write in all methods inside the process), and dynamic variables
(implementing dynamic scope).!

ProcessSpecificVariable class
	instanceVariableNames: 'hash'!

ProcessSpecificVariable subclass: #DynamicVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ProcessSpecific'!

!DynamicVariable commentStamp: 'mvl 3/13/2007 13:55' prior: 0!
My subclasses are dynamic variables: each subclass represents a variable
whose value persists inside the block passed to #value:during:. There is
no way to change the value inside such a block, but it is possible to
temporarirly rebind it in a nested manner.!

!DynamicVariable class methodsFor: 'accessing' stamp: 'bs 2/28/2011 13:25:05.779'!
value: anObject during: aBlock

	| p oldValue |
	
	p := Processor activeProcess.
	oldValue := p environmentAt: self ifAbsent: [self default].
	^ [
		p environmentAt: self put: anObject.
		aBlock value.
	] ensure: [ 
		p environmentAt: self put: oldValue 
	].! !

DynamicVariable subclass: #TestDynamicVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ProcessSpecific-Tests'!

!TestDynamicVariable commentStamp: 'mvl 3/13/2007 13:51' prior: 0!
TestDynamicVariable is a test class using in ProcessSpecificTest.

!

ProcessSpecificVariable subclass: #ProcessLocalVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ProcessSpecific'!

!ProcessLocalVariable commentStamp: 'mvl 3/13/2007 12:28' prior: 0!
My subclasses have values specific to the active process. They can be read with #value and set with #value:!

!ProcessLocalVariable class methodsFor: 'accessing' stamp: 'mvl 3/13/2007 14:27'!
value: anObject
	Processor activeProcess environmentAt: self put: anObject! !

ProcessLocalVariable subclass: #TestLocalVariable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ProcessSpecific-Tests'!

!TestLocalVariable commentStamp: 'mvl 3/13/2007 13:52' prior: 0!
TestLocalVariable is a test class using in ProcessSpecificTest.!

!TestLocalVariable class methodsFor: 'as yet unclassified' stamp: 'mvl 3/13/2007 14:54'!
default
	"My default value for a new process is 0."
	
	^0! !

!ProcessSpecificVariable class methodsFor: 'accessing' stamp: 'mvl 3/13/2007 14:32'!
default
	"Answer the default value for the variable. The default for the default value is nil."
	^nil! !

!ProcessSpecificVariable class methodsFor: 'accessing' stamp: 'mvl 3/13/2007 15:13'!
hash
	hash ifNil:[hash := super hash].
	^hash! !

!ProcessSpecificVariable class methodsFor: 'class initialization' stamp: 'mvl 3/13/2007 16:07'!
initialize
	"Add Process::env if it is missing"
	
	(Process instVarNames includes: 'env')
	ifFalse: [ Process addInstVarName: 'env'].! !

!ProcessSpecificVariable class methodsFor: 'accessing' stamp: 'mvl 3/13/2007 14:33'!
value
	"Answer the current value for this variable in the current context."
	^Processor activeProcess environmentAt: self ifAbsent: [self default].! !
ProcessSpecificVariable initialize!
